# -*- coding: utf-8 -*-

# Import from Django
from django.contrib.admin import ModelAdmin
from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from ckeditor.widgets import CKEditorWidget
from modeltranslation.admin import TranslationAdmin

from crma.models import (
    MailingList,
    Channel,
    Email,
    EmailScheduler,
    Subscription,
    Contact,
)


class EmailForm(forms.ModelForm):
    class Meta:
        model = Email
        widgets = {
            'text': CKEditorWidget(),
        }
        fields = '__all__'


class EmailAdmin(TranslationAdmin):
    form = EmailForm
    list_display = [
        'channel',
        'subject',
        'interval',
        'tag',
        'enabled',
    ]
    list_filter = [
        'channel',
        'enabled',
    ]

    class Media:
        js = [
            'modeltranslation/js/force_jquery.js',
            'js/jquery-ui.js',
            'modeltranslation/js/tabbed_translation_fields.js',
        ]
        css = {
            'screen': [
                'modeltranslation/css/tabbed_translation_fields.css',
            ],
        }


class EmailSchedulerAdmin(ModelAdmin):
    list_display = [
        'from_address',
        'contact',
        'email',
        'lang',
        'ctime',
        'sched_time',
        'sent_time',
        'status',
    ]
    search_fields = [
        'from_address',
        'email__subject',
        'contact__email',
    ]
    list_filter = [
        'status',
        'lang',
        'email',
    ]
    readonly_fields = [
        'extra_context',
        'trace_error',
    ]

    fieldsets = [
        (_('Emails'), {
            'fields': (
                'from_address', 'contact', 'email', 'lang', 'sched_time'
            )}),
        (_('Status'), {
            'fields': (
                ('sent_time', 'status'),
            )}),
        (_('Context'), {
            'fields': (
                ('extra_context', 'trace_error'),
            )}),
    ]


class SubscriptionAdmin(ModelAdmin):
    list_display = [
        'channel',
        'contact',
        'state',
    ]
    search_fields = [
        'channel__title',
        'contact__email',
    ]
    list_filter = [
        'channel',
        'state',
    ]
    fieldsets = [
        (_('Emails'), {
            'fields': (
                'channel', 'contact',
            )}),
        (_('Subscription info'), {
            'fields': (
                ('state', 'unsubscribe_key'),
            )}),
    ]
    readonly_fields = [
        'unsubscribe_key',
    ]


class ChannelAdmin(TranslationAdmin):
    list_display = [
        'title',
        'from_address',
        'required',
    ]
    list_filter = [
        'title',
        'from_address',
        'required',
    ]
    prepopulated_fields = {
        'channel_id': ['title']
    }

    class Media:
        js = [
            'modeltranslation/js/force_jquery.js',
            'js/jquery-ui.js',
            'modeltranslation/js/tabbed_translation_fields.js',
        ]
        css = {
            'screen': [
                'modeltranslation/css/tabbed_translation_fields.css',
            ],
        }


class ContactAdmin(ModelAdmin):
    list_display = [
        'email',
        'lang',
    ]
    list_filter = [
        'lang',
    ]
    search_fields = [
        'email',
    ]


class MembersInline(admin.TabularInline):
    model = MailingList.members.through
    extra = 0
    raw_id_fields = [
        'contact',
    ]


class MailingListForm(forms.ModelForm):
    csv = forms.FileField(
        required=False,
        help_text=_(u'The CSV file must have a header line with two columns: "email" and "lang".')
    )

    class Meta:
        model = MailingList
        fields = [
            'title',
            'all',
            'members',
        ]


class MailingListAdmin(ModelAdmin):
    list_display = [
        'title',
    ]
    filter_horizontal = [
        'members',
    ]
    form = MailingListForm
    exclude = [
        'members',
    ]

    inlines = [
        MembersInline,
    ]

    def save_related(self, request, form, formsets, change):
        proxy = super(MailingListAdmin, self)
        proxy.save_related(request, form, formsets, change)

        csv = form.cleaned_data['csv']
        if csv:
            obj = form.instance
            import_contacts(obj, csv)


def _csv_get_rows(f):
    import csv

    reader = csv.DictReader(f, restval='')
    for row in reader:
        yield row['email'], row.get('lang', '')


def _xls_get_rows(data):
    from xlrd import open_workbook

    wb = open_workbook(file_contents=data)
    rows = wb.sheet_by_index(0).get_rows()
    header = [x.value for x in rows.next()]
    i = header.index('email')
    j = header.index('lang')

    for row in rows:
        yield row[i].value, row[j].value


def import_contacts(mailing_list, input_file):
    # Detect file type
    import magic

    with magic.Magic() as m:
        data = input_file.read()
        mimetype = m.from_buffer(data)

    if mimetype.startswith('text/'):
        input_file.seek(0)
        rows = _csv_get_rows(input_file)
    else:
        rows = _xls_get_rows(data)

    for email, lang in rows:
        try:
            contact = Contact.objects.get(email=email)
        except Contact.DoesNotExist:
            contact = Contact.objects.create(email=email, lang=lang)
        else:
            if lang and lang != contact.lang:
                contact.lang = lang
                contact.save()

        mailing_list.members.add(contact)


try:
    from import_export.admin import ImportExportMixin
except ImportError:
    pass
else:
    from crma.resources import ContactResource
    ContactAdmin = type(ContactAdmin)('ContactAdmin', (ImportExportMixin, ContactAdmin,), {
        'resource_class': ContactResource,
    })

admin.site.register(Channel, ChannelAdmin)
admin.site.register(Email, EmailAdmin)
admin.site.register(EmailScheduler, EmailSchedulerAdmin)
admin.site.register(Subscription, SubscriptionAdmin)
admin.site.register(Contact, ContactAdmin)
admin.site.register(MailingList, MailingListAdmin)
