# -*- coding: utf-8 -*-

import bleach

from django.core.mail import EmailMultiAlternatives
from django.template import Context, Template
from django.utils.translation import activate
from django.utils.functional import cached_property
from django.core.urlresolvers import reverse
from django.conf import settings

from crma.utils import encode_id
from crma.models import EmailScheduler


def get_lang_code(lang):
    if not lang:
        return None

    languages = [x[0].lower() for x in settings.LANGUAGES]
    languages.sort(reverse=True)

    lang = lang.lower()
    for l in languages:
        if lang.startswith(l):
            return l

    return None


class MailBuilder(object):

    def __init__(self, scheduler, site):
        assert isinstance(scheduler, EmailScheduler), TypeError('%r is not an EmailScheduler' % scheduler)
        self.scheduler = scheduler
        self.email_template = scheduler.email
        self.site = site

    @cached_property
    def language(self):
        lang = get_lang_code(self.scheduler.lang)
        if not lang:
            return 'en'

        subject = getattr(self.email_template, 'subject_{}'.format(lang), None) or ''

        if subject.strip():
            return lang
        return 'en'

    @cached_property
    def subscription(self):
        return self.scheduler.get_subscription()

    def should_send_email(self):
        return self.subscription.is_active

    @cached_property
    def context(self):
        # Generate unsubscribe url
        unsubscribe_path = reverse('crma_unsubscribe', args=[self.subscription.unsubscribe_key])
        unsubscribe_url = self.build_url(unsubscribe_path)

        viewmail_path = reverse('crma_view_web_mail', args=[encode_id(self.scheduler.pk),
                                                            self.scheduler.contact.generate_token()])
        viewmail_url = self.build_url(viewmail_path)

        context = {
            'contact': self.scheduler.contact,
            'LANG': self.language,
            'UNSUBSCRIBE_URL': unsubscribe_url,
            'VIEWMAIL_URL': viewmail_url,
        }
        context.update(self.scheduler.extra_context)
        return context

    def build_email(self):
        """
        - contact: Contact instance
        - lang: Language code
        - from_name: Name to display in the email address
        - extra_context: dict with variables to add to the template context

        return False is the mail is not sent else True
        """

        subject, body_html = self.render_html()
        body_text = bleach.clean(body_html)

        email_to = [self.scheduler.contact.email]
        email_from = self.email_template.channel.from_address

        headers = {'X-Tag': self.email_template.tag}
        msg = EmailMultiAlternatives(subject, body_text, email_from, email_to, headers=headers)
        msg.attach_alternative(body_html, 'text/html')
        return msg

    def build_url(self, path):
        return 'http{secure}://{domain}{path}'.format(
            secure='s' if getattr(settings, 'CRMA_SECURE', True) else '',
            domain=self.site.domain,
            path=path
        )

    def render_html(self):
        # Generate Subject and body content
        activate(self.language)
        subject = self.email_template.subject
        body_html = self.email_template.body

        context = Context(self.context)
        body = Template(body_html).render(context)
        subject = Template(subject).render(context)
        return subject, body
